@@ -27,7 +27,7 @@ func LoadEbpfModule(module string) (m *elf.Module, err error) {
 			log.Info("[eBPF] module loaded: %s/%s", modulesPath, module)
 			return m, nil
 		}
-		log.Debug("ebpf module not loaded: %s, %s/%s", err, modulesPath, module)
+		log.Debug("ebpf module not found: %s, %s/%s", err, modulesPath, module)
 	}
 
 	return m, fmt.Errorf(`
