@@ -14,6 +14,7 @@ const (
 // Verdict holds the action to perform on a packet (NF_DROP, NF_ACCEPT, etc)
 type Verdict C.uint
 
+// VerdictContainer struct
 type VerdictContainer struct {
 	Verdict Verdict
 	Mark    uint32
@@ -27,6 +28,8 @@ type Packet struct {
 	verdictChannel  chan VerdictContainer
 	UID             uint32
 	NetworkProtocol uint8
+	IfaceInIdx      int
+	IfaceOutIdx     int
 }
 
 // SetVerdict emits a veredict on a packet
@@ -40,13 +43,15 @@ func (p *Packet) SetVerdictAndMark(v Verdict, mark uint32) {
 	p.verdictChannel <- VerdictContainer{Verdict: v, Packet: nil, Mark: mark}
 }
 
-func (p *Packet) SetRequeueVerdict(newQueueId uint16) {
+// SetRequeueVerdict apply a verdict on a requeued packet
+func (p *Packet) SetRequeueVerdict(newQueueID uint16) {
 	v := uint(NF_QUEUE)
-	q := (uint(newQueueId) << 16)
+	q := (uint(newQueueID) << 16)
 	v = v | q
 	p.verdictChannel <- VerdictContainer{Verdict: Verdict(v), Packet: nil, Mark: 0}
 }
 
+// SetVerdictWithPacket apply a verdict, but with a new packet
 func (p *Packet) SetVerdictWithPacket(v Verdict, packet []byte) {
 	p.verdictChannel <- VerdictContainer{Verdict: v, Packet: packet, Mark: 0}
 }
