@@ -16,7 +16,7 @@ struct tcp_key_t {
 struct tcp_value_t {
 	pid_size_t pid;
 	uid_size_t uid;
-	u64 counter;
+	char comm[TASK_COMM_LEN];
 }__attribute__((packed));
 
 // not using unsigned __int128 because it is not supported on x86_32
@@ -35,7 +35,7 @@ struct tcpv6_key_t {
 struct tcpv6_value_t{
 	pid_size_t pid;
 	uid_size_t uid;
-	u64 counter; 
+	char comm[TASK_COMM_LEN];
 }__attribute__((packed));
 
 struct udp_key_t {
@@ -48,7 +48,7 @@ struct udp_key_t {
 struct udp_value_t{
 	pid_size_t pid;
 	uid_size_t uid;
-	u64 counter; 
+	char comm[TASK_COMM_LEN];
 }__attribute__((packed));
 
 struct udpv6_key_t {
@@ -61,7 +61,7 @@ struct udpv6_key_t {
 struct udpv6_value_t{
 	pid_size_t pid;
 	uid_size_t uid;
-	u64 counter;
+	char comm[TASK_COMM_LEN];
 }__attribute__((packed));
 
 
@@ -116,38 +116,6 @@ struct bpf_map_def SEC("maps/tcpv6sock") tcpv6sock = {
 	.max_entries = 100,
 };
 
-// counts how many connections we've processed. Starts at 0.
-struct bpf_map_def SEC("maps/tcpcounter") tcpcounter = {
-	.type = BPF_MAP_TYPE_ARRAY,
-	.key_size = sizeof(u32),
-	.value_size = sizeof(u64),
-	.max_entries = 1,
-};
-struct bpf_map_def SEC("maps/tcpv6counter") tcpv6counter = {
-	.type = BPF_MAP_TYPE_ARRAY,
-	.key_size = sizeof(u32),
-	.value_size = sizeof(u64),
-	.max_entries = 1,
-};
-struct bpf_map_def SEC("maps/udpcounter") udpcounter = {
-	.type = BPF_MAP_TYPE_ARRAY,
-	.key_size = sizeof(u32),
-	.value_size = sizeof(u64),
-	.max_entries = 1,
-};
-struct bpf_map_def SEC("maps/udpv6counter") udpv6counter = {
-	.type = BPF_MAP_TYPE_ARRAY,
-	.key_size = sizeof(u32),
-	.value_size = sizeof(u64),
-	.max_entries = 1,
-};
-struct bpf_map_def SEC("maps/debugcounter") debugcounter = {
-	.type = BPF_MAP_TYPE_ARRAY,
-	.key_size = sizeof(u32),
-	.value_size = sizeof(u64),
-	.max_entries = 1,
-};
-
 // size 150 gave ebpf verifier errors for kernel 4.14, 100 is ok
 // we can cast any struct into rawBytes_t to be able to access arbitrary bytes of the struct 
 struct rawBytes_t {
@@ -188,7 +156,7 @@ int kprobe__tcp_v4_connect(struct pt_regs *ctx)
 
 	u64 skp = (u64)sk;
 	u64 pid_tgid = bpf_get_current_pid_tgid();
-    bpf_map_update_elem(&tcpsock, &pid_tgid, &skp, BPF_ANY);
+	bpf_map_update_elem(&tcpsock, &pid_tgid, &skp, BPF_ANY);
 	return 0;
 };
 
@@ -204,25 +172,19 @@ int kretprobe__tcp_v4_connect(struct pt_regs *ctx)
 	sk = (struct sock *)*skp;
 
 	struct tcp_key_t tcp_key;
-    __builtin_memset(&tcp_key, 0, sizeof(tcp_key));
+	__builtin_memset(&tcp_key, 0, sizeof(tcp_key));
 	bpf_probe_read(&tcp_key.dport, sizeof(tcp_key.dport), &sk->__sk_common.skc_dport);
 	bpf_probe_read(&tcp_key.sport, sizeof(tcp_key.sport), &sk->__sk_common.skc_num);
 	bpf_probe_read(&tcp_key.daddr, sizeof(tcp_key.daddr), &sk->__sk_common.skc_daddr);
 	bpf_probe_read(&tcp_key.saddr, sizeof(tcp_key.saddr), &sk->__sk_common.skc_rcv_saddr);
 	
-	u32 zero_key = 0;
-	u64 *val = bpf_map_lookup_elem(&tcpcounter, &zero_key);
-	if (val == NULL){return 0;}
-
-	struct tcp_value_t tcp_value;
+	struct tcp_value_t tcp_value={0};
 	__builtin_memset(&tcp_value, 0, sizeof(tcp_value));
 	tcp_value.pid = pid_tgid >> 32;
 	tcp_value.uid = bpf_get_current_uid_gid() & 0xffffffff;
-	tcp_value.counter = *val;
+	bpf_get_current_comm(&tcp_value.comm, sizeof(tcp_value.comm));
 	bpf_map_update_elem(&tcpMap, &tcp_key, &tcp_value, BPF_ANY);
 
-	u64 newval = *val + 1;
-	bpf_map_update_elem(&tcpcounter, &zero_key, &newval, BPF_ANY);
 	bpf_map_delete_elem(&tcpsock, &pid_tgid);
 	return 0;
 };
@@ -254,13 +216,13 @@ int kretprobe__tcp_v6_connect(struct pt_regs *ctx)
 	sk = (struct sock *)*skp;
 	
 	struct tcpv6_key_t tcpv6_key;
-    __builtin_memset(&tcpv6_key, 0, sizeof(tcpv6_key));
+	__builtin_memset(&tcpv6_key, 0, sizeof(tcpv6_key));
 	bpf_probe_read(&tcpv6_key.dport, sizeof(tcpv6_key.dport), &sk->__sk_common.skc_dport);
 	bpf_probe_read(&tcpv6_key.sport, sizeof(tcpv6_key.sport), &sk->__sk_common.skc_num);
 #if defined(__i386__)
 		struct sock_on_x86_32_t sock;
-    __builtin_memset(&sock, 0, sizeof(sock));
-    bpf_probe_read(&sock, sizeof(sock), *(&sk));
+		__builtin_memset(&sock, 0, sizeof(sock));
+		bpf_probe_read(&sock, sizeof(sock), *(&sk));
 		tcpv6_key.daddr = sock.daddr;
 		tcpv6_key.saddr = sock.saddr;
 #else
@@ -268,19 +230,13 @@ int kretprobe__tcp_v6_connect(struct pt_regs *ctx)
 		bpf_probe_read(&tcpv6_key.saddr, sizeof(tcpv6_key.saddr), &sk->__sk_common.skc_v6_rcv_saddr.in6_u.u6_addr32);
 #endif
 
-	u32 zero_key = 0;
-	u64 *val = bpf_map_lookup_elem(&tcpv6counter, &zero_key);
-	if (val == NULL){return 0;}
-
-	struct tcpv6_value_t tcpv6_value;
-    __builtin_memset(&tcpv6_value, 0, sizeof(tcpv6_value));
+	struct tcpv6_value_t tcpv6_value={0};
+	__builtin_memset(&tcpv6_value, 0, sizeof(tcpv6_value));
 	tcpv6_value.pid = pid_tgid >> 32;
 	tcpv6_value.uid = bpf_get_current_uid_gid() & 0xffffffff;
-	tcpv6_value.counter = *val;
+	bpf_get_current_comm(&tcpv6_value.comm, sizeof(tcpv6_value.comm));
 	bpf_map_update_elem(&tcpv6Map, &tcpv6_key, &tcpv6_value, BPF_ANY);
 
-	u64 newval = *val + 1;
-	bpf_map_update_elem(&tcpv6counter, &zero_key, &newval, BPF_ANY);
 	bpf_map_delete_elem(&tcpv6sock, &pid_tgid);
 	return 0;
 };
@@ -297,12 +253,12 @@ int kprobe__udp_sendmsg(struct pt_regs *ctx)
 #endif
 
 	u64 msg_name; //pointer
-    __builtin_memset(&msg_name, 0, sizeof(msg_name));
+	__builtin_memset(&msg_name, 0, sizeof(msg_name));
 	bpf_probe_read(&msg_name, sizeof(msg_name), &msg->msg_name);
 	struct sockaddr_in * usin = (struct sockaddr_in *)msg_name;
 
 	struct udp_key_t udp_key;
-    __builtin_memset(&udp_key, 0, sizeof(udp_key));
+	__builtin_memset(&udp_key, 0, sizeof(udp_key));
 	bpf_probe_read(&udp_key.dport, sizeof(udp_key.dport), &usin->sin_port);
 	if (udp_key.dport != 0){ //likely
 		bpf_probe_read(&udp_key.daddr, sizeof(udp_key.daddr), &usin->sin_addr.s_addr);
@@ -316,21 +272,16 @@ int kprobe__udp_sendmsg(struct pt_regs *ctx)
 	bpf_probe_read(&udp_key.saddr, sizeof(udp_key.saddr), &sk->__sk_common.skc_rcv_saddr);
 	
 	u32 zero_key = 0;
-    __builtin_memset(&zero_key, 0, sizeof(zero_key));
-	u64 *counterVal = bpf_map_lookup_elem(&udpcounter, &zero_key);
-	if (counterVal == NULL){return 0;}
+	__builtin_memset(&zero_key, 0, sizeof(zero_key));
 	struct udp_value_t *lookedupValue = bpf_map_lookup_elem(&udpMap, &udp_key);
 	u64 pid = bpf_get_current_pid_tgid() >> 32;
 	if ( lookedupValue == NULL || lookedupValue->pid != pid) {
 		struct udp_value_t udp_value={0};
-        __builtin_memset(&udp_value, 0, sizeof(udp_value));
+       		__builtin_memset(&udp_value, 0, sizeof(udp_value));
 		udp_value.pid = pid;
 		udp_value.uid = bpf_get_current_uid_gid() & 0xffffffff;
-		udp_value.counter = *counterVal;
+		bpf_get_current_comm(&udp_value.comm, sizeof(udp_value.comm));
 		bpf_map_update_elem(&udpMap, &udp_key, &udp_value, BPF_ANY);
-
-		u64 newval = *counterVal + 1;
-		bpf_map_update_elem(&udpcounter, &zero_key, &newval, BPF_ANY);
 	}
 	//else nothing to do
 	return 0;
@@ -349,11 +300,11 @@ int kprobe__udpv6_sendmsg(struct pt_regs *ctx)
 #endif
 
 	u64 msg_name; //a pointer
-    __builtin_memset(&msg_name, 0, sizeof(msg_name));
+	__builtin_memset(&msg_name, 0, sizeof(msg_name));
 	bpf_probe_read(&msg_name, sizeof(msg_name), &msg->msg_name);
 
 	struct udpv6_key_t udpv6_key;
-    __builtin_memset(&udpv6_key, 0, sizeof(udpv6_key));
+	__builtin_memset(&udpv6_key, 0, sizeof(udpv6_key));
 	bpf_probe_read(&udpv6_key.dport, sizeof(udpv6_key.dport), &sk->__sk_common.skc_dport);
 	if (udpv6_key.dport != 0){ //likely
 		bpf_probe_read(&udpv6_key.daddr, sizeof(udpv6_key.daddr), &sk->__sk_common.skc_v6_daddr.in6_u.u6_addr32);
@@ -377,19 +328,15 @@ int kprobe__udpv6_sendmsg(struct pt_regs *ctx)
 #endif
 
 	u32 zero_key = 0;
-	u64 *counterVal = bpf_map_lookup_elem(&udpv6counter, &zero_key);
-	if (counterVal == NULL){return 0;}
 	struct udpv6_value_t *lookedupValue = bpf_map_lookup_elem(&udpv6Map, &udpv6_key);
 	u64 pid = bpf_get_current_pid_tgid() >> 32;
 	if ( lookedupValue == NULL || lookedupValue->pid != pid) {
 		struct udpv6_value_t udpv6_value={0};
-        __builtin_memset(&udpv6_value, 0, sizeof(udpv6_value));
+		__builtin_memset(&udpv6_value, 0, sizeof(udpv6_value));
+		bpf_get_current_comm(&udpv6_value.comm, sizeof(udpv6_value.comm));
 		udpv6_value.pid = pid;
 		udpv6_value.uid = bpf_get_current_uid_gid() & 0xffffffff;
-		udpv6_value.counter = *counterVal;
 		bpf_map_update_elem(&udpv6Map, &udpv6_key, &udpv6_value, BPF_ANY);
-		u64 newval = *counterVal + 1;
-		bpf_map_update_elem(&udpv6counter, &zero_key, &newval, BPF_ANY);	
 	}
 	//else nothing to do
 	return 0;
@@ -431,20 +378,16 @@ int kprobe__iptunnel_xmit(struct pt_regs *ctx)
 	bpf_probe_read(&udp_key.saddr, sizeof(udp_key.saddr), &src);
 	bpf_probe_read(&udp_key.daddr, sizeof(udp_key.daddr), &dst);
 
-	u64 *counterVal = bpf_map_lookup_elem(&udpcounter, &zero_key);
-	if (counterVal == NULL){return 0;}
-
 	struct udp_value_t *lookedupValue = bpf_map_lookup_elem(&udpMap, &udp_key);
 	u64 pid = bpf_get_current_pid_tgid() >> 32;
 	if ( lookedupValue == NULL || lookedupValue->pid != pid) {
+		bpf_get_current_comm(&udp_value.comm, sizeof(udp_value.comm));
 		udp_value.pid = pid;
 		udp_value.uid = bpf_get_current_uid_gid() & 0xffffffff;
-		udp_value.counter = *counterVal;
 		bpf_map_update_elem(&udpMap, &udp_key, &udp_value, BPF_ANY);
-		u64 newval = *counterVal + 1;
-		bpf_map_update_elem(&udpcounter, &zero_key, &newval, BPF_ANY);
 	}
 
+	//else nothing to do
 	return 0;
 };
 #endif
