@@ -8,10 +8,11 @@ import json
 import ipaddress
 
 from PyQt5 import QtCore, QtGui, uic, QtWidgets
-from PyQt5.QtCore import QCoreApplication as QC
+from PyQt5.QtCore import QCoreApplication as QC, QEvent
 
 from slugify import slugify
 
+from opensnitch.utils import Icons
 from opensnitch.desktop_parser import LinuxDesktopParser
 from opensnitch.config import Config
 from opensnitch.version import version
@@ -26,9 +27,7 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
 
     DEFAULT_TIMEOUT = 15
 
-    ACTION_IDX_DENY = 0
-    ACTION_IDX_ALLOW = 1
-
+    # don't translate
     FIELD_REGEX_HOST    = "regex_host"
     FIELD_REGEX_IP      = "regex_ip"
     FIELD_PROC_PATH     = "process_path"
@@ -40,7 +39,6 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
     FIELD_DST_NETWORK   = "dst_network"
     FIELD_DST_HOST      = "simple_host"
 
-    # don't translate
     DURATION_30s    = "30s"
     DURATION_5m     = "5m"
     DURATION_15m    = "15m"
@@ -59,6 +57,7 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         self._cfg = Config.get()
         self.setupUi(self)
         self.setWindowIcon(appicon)
+        self.installEventFilter(self)
 
         self._width = None
         self._height = None
@@ -85,13 +84,6 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
 
         self._apps_parser = LinuxDesktopParser()
 
-        self.denyButton.clicked.connect(self._on_deny_clicked)
-        # also accept button
-        self.applyButton.clicked.connect(self._on_apply_clicked)
-        self._apply_text = QC.translate("popups", "Allow")
-        self._deny_text = QC.translate("popups", "Deny")
-        self._default_action = self._cfg.getInt(self._cfg.DEFAULT_ACTION_KEY)
-
         self.whatIPCombo.setVisible(False)
         self.checkDstIP.setVisible(False)
         self.checkDstPort.setVisible(False)
@@ -109,21 +101,58 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         self.checkDstPort.clicked.connect(self._button_clicked)
         self.checkUserID.clicked.connect(self._button_clicked)
 
-        if QtGui.QIcon.hasThemeIcon("emblem-default") == False:
-            self.applyButton.setIcon(self.style().standardIcon(getattr(QtWidgets.QStyle, "SP_DialogApplyButton")))
-            self.denyButton.setIcon(self.style().standardIcon(getattr(QtWidgets.QStyle, "SP_DialogCancelButton")))
+        self.allowIcon = Icons.new("emblem-default")
+        denyIcon = Icons.new("emblem-important")
+        rejectIcon = Icons.new("window-close")
+
+        self._default_action = self._cfg.getInt(self._cfg.DEFAULT_ACTION_KEY)
+
+        self.allowButton.clicked.connect(lambda: self._on_action_clicked(Config.ACTION_ALLOW_IDX))
+        self.allowButton.setIcon(self.allowIcon)
+        self._allow_text = QC.translate("popups", "Allow")
+        self._action_text = [
+            QC.translate("popups", "Deny"),
+            QC.translate("popups", "Allow"),
+            QC.translate("popups", "Reject")
+        ]
+        self._action_icon = [denyIcon, self.allowIcon, rejectIcon]
+
+        m = QtWidgets.QMenu()
+        m.addAction(denyIcon, self._action_text[Config.ACTION_DENY_IDX]).triggered.connect(
+            lambda: self._on_action_clicked(Config.ACTION_DENY_IDX)
+        )
+        m.addAction(self.allowIcon, self._action_text[Config.ACTION_ALLOW_IDX]).triggered.connect(
+            lambda: self._on_action_clicked(Config.ACTION_ALLOW_IDX)
+        )
+        m.addAction(rejectIcon, self._action_text[Config.ACTION_REJECT_IDX]).triggered.connect(
+            lambda: self._on_action_clicked(Config.ACTION_REJECT_IDX)
+        )
+        self.actionButton.setMenu(m)
+        self.actionButton.setText(self._action_text[Config.ACTION_DENY_IDX])
+        self.actionButton.setIcon(self._action_icon[Config.ACTION_DENY_IDX])
+        if self._default_action != Config.ACTION_ALLOW_IDX:
+            self.actionButton.setText(self._action_text[self._default_action])
+            self.actionButton.setIcon(self._action_icon[self._default_action])
+        self.actionButton.clicked.connect(self._on_deny_btn_clicked)
+
+    def eventFilter(self, obj, event):
+        if event.type() == QEvent.MouseButtonPress:
+            self._stop_countdown()
+            return True
+        return False
 
     def showEvent(self, event):
         super(PromptDialog, self).showEvent(event)
         self.activateWindow()
+        self.adjust_size()
+        self.move_popup()
 
+    def adjust_size(self):
         if self._width is None or self._height is None:
             self._width = self.width()
             self._height = self.height()
 
-        self.setMinimumSize(self._width, self._height)
-        self.setMaximumSize(self._width, self._height)
-        self.move_popup()
+        self.resize(QtCore.QSize(self._width, self._height))
 
     def move_popup(self):
         popup_pos = self._cfg.getInt(self._cfg.DEFAULT_POPUP_POSITION)
@@ -138,17 +167,25 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             self.move(point.bottomLeft())
 
     def _stop_countdown(self):
-        self.applyButton.setText("%s" % self._apply_text)
-        self.denyButton.setText("%s" % self._deny_text)
+        action_idx = self._cfg.getInt(self._cfg.DEFAULT_ACTION_KEY)
+        if action_idx == Config.ACTION_ALLOW_IDX:
+            self.allowButton.setText(self._allow_text)
+            self.allowButton.setIcon(self.allowIcon)
+        else:
+            self.actionButton.setText(self._action_text[action_idx])
+            self.actionButton.setIcon(self._action_icon[action_idx])
         self._tick_thread.stop = True
 
     def _check_advanced_toggled(self, state):
         self.checkDstIP.setVisible(state)
         self.whatIPCombo.setVisible(state)
         self.destIPLabel.setVisible(not state)
-        self.checkDstPort.setVisible(state)
+        self.checkDstPort.setVisible(state == True and (self._con != None and self._con.dst_port != 0))
         self.checkUserID.setVisible(state)
+
         self._ischeckAdvanceded = state
+        self.adjust_size()
+        self.move_popup()
 
     def _button_clicked(self):
         self._stop_countdown()
@@ -219,6 +256,9 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         self._timeout_triggered = True
         self._send_rule()
 
+    def _hide_widget(self, widget, hide):
+        widget.setVisible(not hide)
+
     def _configure_default_duration(self):
         if self._cfg.hasKey(self._cfg.DEFAULT_DURATION_KEY):
             cur_idx = self._cfg.getInt(self._cfg.DEFAULT_DURATION_KEY)
@@ -227,12 +267,15 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             self.durationCombo.setCurrentIndex(self._cfg.DEFAULT_DURATION_IDX)
 
     def _set_cmd_action_text(self):
-        if self._cfg.getInt(self._cfg.DEFAULT_ACTION_KEY) == self.ACTION_IDX_ALLOW:
-            self.applyButton.setText("%s (%d)" % (self._apply_text, self._tick))
-            self.denyButton.setText(self._deny_text)
+        action_idx = self._cfg.getInt(self._cfg.DEFAULT_ACTION_KEY)
+        if action_idx == Config.ACTION_ALLOW_IDX:
+            self.allowButton.setText("{0} ({1})".format(self._allow_text, self._tick))
+            self.allowButton.setIcon(self.allowIcon)
+            self.actionButton.setText(self._action_text[Config.ACTION_DENY_IDX])
         else:
-            self.denyButton.setText("%s (%d)" % (self._deny_text, self._tick))
-            self.applyButton.setText(self._apply_text)
+            self.allowButton.setText(self._allow_text)
+            self.actionButton.setText("{0} ({1})".format(self._action_text[action_idx], self._tick))
+            self.actionButton.setIcon(self._action_icon[action_idx])
 
     def _set_app_description(self, description):
         if description != None and description != "":
@@ -256,6 +299,9 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             else:
                 self._set_elide_text(self.appPathLabel, "(%s)" % con.process_path)
             self.appPathLabel.setVisible(True)
+        elif con.process_path != "" and len(con.process_args) == 0:
+            self._set_elide_text(self.appPathLabel, "%s" % con.process_path)
+            self.appPathLabel.setVisible(True)
         else:
             self.appPathLabel.setVisible(False)
             self.appPathLabel.setText("")
@@ -300,7 +346,13 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
 
         self.sourceIPLabel.setText(con.src_ip)
         self.destIPLabel.setText(con.dst_ip)
-        self.destPortLabel.setText(str(con.dst_port))
+        if con.dst_port == 0:
+            self.destPortLabel.setText("")
+        else:
+            self.destPortLabel.setText(str(con.dst_port))
+        self._hide_widget(self.destPortLabel, con.dst_port == 0)
+        self._hide_widget(self.destPortLabel_1, con.dst_port == 0)
+        self._hide_widget(self.checkDstPort, con.dst_port == 0 or not self._ischeckAdvanceded)
 
         if self._local:
             try:
@@ -440,6 +492,12 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             con.protocol.upper(),
             con.dst_port )
 
+        # icmp port is 0 (i.e.: no port)
+        if con.dst_port == 0:
+            msg_action = QC.translate("popups", "is connecting to <b>%s</b>, %s") % ( \
+                con.dst_host or con.dst_ip,
+                con.protocol.upper() )
+
         if con.dst_port == 53 and con.dst_ip != con.dst_host and con.dst_host != "":
             msg_action = QC.translate("popups", "is attempting to resolve <b>%s</b> via %s, %s port %d") % ( \
                 con.dst_host,
@@ -469,35 +527,35 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
 
     def _get_combo_operator(self, combo, what_idx):
         if combo.itemData(what_idx) == self.FIELD_PROC_PATH:
-            return Config.RULE_TYPE_SIMPLE, "process.path", self._con.process_path
+            return Config.RULE_TYPE_SIMPLE, Config.OPERAND_PROCESS_PATH, self._con.process_path
 
         elif combo.itemData(what_idx) == self.FIELD_PROC_ARGS:
             # this should not happen
             if len(self._con.process_args) == 0 or self._con.process_args[0] == "":
-                return Config.RULE_TYPE_SIMPLE, "process.path", self._con.process_path
-            return Config.RULE_TYPE_SIMPLE, "process.command", ' '.join(self._con.process_args)
+                return Config.RULE_TYPE_SIMPLE, Config.OPERAND_PROCESS_PATH, self._con.process_path
+            return Config.RULE_TYPE_SIMPLE, Config.OPERAND_PROCESS_COMMAND, ' '.join(self._con.process_args)
 
         elif combo.itemData(what_idx) == self.FIELD_PROC_ID:
-            return Config.RULE_TYPE_SIMPLE, "process.id", "{0}".format(self._con.process_id)
+            return Config.RULE_TYPE_SIMPLE, Config.OPERAND_PROCESS_ID, "{0}".format(self._con.process_id)
 
         elif combo.itemData(what_idx) == self.FIELD_USER_ID:
-            return Config.RULE_TYPE_SIMPLE, "user.id", "%s" % self._con.user_id
+            return Config.RULE_TYPE_SIMPLE, Config.OPERAND_USER_ID, "%s" % self._con.user_id
 
         elif combo.itemData(what_idx) == self.FIELD_DST_PORT:
-            return Config.RULE_TYPE_SIMPLE, "dest.port", "%s" % self._con.dst_port
+            return Config.RULE_TYPE_SIMPLE, Config.OPERAND_DEST_PORT, "%s" % self._con.dst_port
 
         elif combo.itemData(what_idx) == self.FIELD_DST_IP:
-            return Config.RULE_TYPE_SIMPLE, "dest.ip", self._con.dst_ip
+            return Config.RULE_TYPE_SIMPLE, Config.OPERAND_DEST_IP, self._con.dst_ip
 
         elif combo.itemData(what_idx) == self.FIELD_DST_HOST:
-            return Config.RULE_TYPE_SIMPLE, "dest.host", combo.currentText()
+            return Config.RULE_TYPE_SIMPLE, Config.OPERAND_DEST_HOST, combo.currentText()
 
         elif combo.itemData(what_idx) == self.FIELD_DST_NETWORK:
             # strip "to ": "to x.x.x/20" -> "x.x.x/20"
             # we assume that to is one word in all languages
             parts = combo.currentText().split(' ')
             text = parts[len(parts)-1]
-            return Config.RULE_TYPE_NETWORK, "dest.network", text
+            return Config.RULE_TYPE_NETWORK, Config.OPERAND_DEST_NETWORK, text
 
         elif combo.itemData(what_idx) == self.FIELD_REGEX_HOST:
             parts = combo.currentText().split(' ')
@@ -505,19 +563,21 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             # ^(|.*\.)yahoo\.com
             dsthost = '\.'.join(text.split('.')).replace("*", "")
             dsthost = "^(|.*\.)%s" % dsthost[2:]
-            return Config.RULE_TYPE_REGEXP, "dest.host", dsthost
+            return Config.RULE_TYPE_REGEXP, Config.OPERAND_DEST_HOST, dsthost
 
         elif combo.itemData(what_idx) == self.FIELD_REGEX_IP:
             parts = combo.currentText().split(' ')
             text = parts[len(parts)-1]
-            return Config.RULE_TYPE_REGEXP, "dest.ip", "%s" % '\.'.join(text.split('.')).replace("*", ".*")
+            return Config.RULE_TYPE_REGEXP, Config.OPERAND_DEST_IP, "%s" % '\.'.join(text.split('.')).replace("*", ".*")
 
-    def _on_deny_clicked(self):
-        self._default_action = self.ACTION_IDX_DENY
+    def _on_action_clicked(self, action):
+        self._default_action = action
         self._send_rule()
 
-    def _on_apply_clicked(self):
-        self._default_action = self.ACTION_IDX_ALLOW
+    def _on_deny_btn_clicked(self, action):
+        self._default_action = self._cfg.getInt(self._cfg.DEFAULT_ACTION_KEY)
+        if self._default_action == Config.ACTION_ALLOW_IDX:
+            self._default_action = Config.ACTION_DENY_IDX
         self._send_rule()
 
     def _is_list_rule(self):
@@ -538,9 +598,14 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             self._cfg.setSettings("promptDialog/geometry", self.saveGeometry())
             self._rule = ui_pb2.Rule(name="user.choice")
             self._rule.enabled = True
-            self._rule.action = Config.ACTION_DENY if self._default_action == self.ACTION_IDX_DENY else Config.ACTION_ALLOW
             self._rule.duration = self._get_duration(self.durationCombo.currentIndex())
 
+            self._rule.action = Config.ACTION_ALLOW
+            if self._default_action == Config.ACTION_DENY_IDX:
+                self._rule.action = Config.ACTION_DENY
+            elif self._default_action == Config.ACTION_REJECT_IDX:
+                self._rule.action = Config.ACTION_REJECT
+
             what_idx = self.whatCombo.currentIndex()
             self._rule.operator.type, self._rule.operator.operand, self._rule.operator.data = self._get_combo_operator(self.whatCombo, what_idx)
             if self._rule.operator.data == "":
@@ -559,14 +624,26 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
                 rule_temp_name = slugify("%s %s" % (rule_temp_name, _data))
 
             if self.checkDstPort.isChecked() and self.whatCombo.itemData(what_idx) != self.FIELD_DST_PORT:
-                data.append({"type": Config.RULE_TYPE_SIMPLE, "operand": "dest.port", "data": str(self._con.dst_port)})
+                data.append({"type": Config.RULE_TYPE_SIMPLE, "operand": Config.OPERAND_DEST_PORT, "data": str(self._con.dst_port)})
                 rule_temp_name = slugify("%s %s" % (rule_temp_name, str(self._con.dst_port)))
 
             if self.checkUserID.isChecked() and self.whatCombo.itemData(what_idx) != self.FIELD_USER_ID:
-                data.append({"type": Config.RULE_TYPE_SIMPLE, "operand": "user.id", "data": str(self._con.user_id)})
+                data.append({"type": Config.RULE_TYPE_SIMPLE, "operand": Config.OPERAND_USER_ID, "data": str(self._con.user_id)})
                 rule_temp_name = slugify("%s %s" % (rule_temp_name, str(self._con.user_id)))
 
-            if self._is_list_rule():
+            is_list_rule = self._is_list_rule()
+
+            # If the user has selected to filter by cmdline, but the launched
+            # command path is not absolute, we can't trust it. In this case,
+            # also filter by the absolute path to the binary.
+            if self._rule.operator.operand == Config.OPERAND_PROCESS_COMMAND:
+                proc_args = " ".join(self._con.process_args)
+                proc_args = proc_args.split(" ")
+                if os.path.isabs(proc_args[0]) == False:
+                    is_list_rule = True
+                    data.append({"type": Config.RULE_TYPE_SIMPLE, "operand": Config.OPERAND_PROCESS_PATH, "data": str(self._con.process_path)})
+
+            if is_list_rule:
                 data.append({"type": self._rule.operator.type, "operand": self._rule.operator.operand, "data": self._rule.operator.data})
                 self._rule.operator.data = json.dumps(data)
                 self._rule.operator.type = Config.RULE_TYPE_LIST
