@@ -7,6 +7,7 @@ import re
 import json
 import sys
 import os
+import pwd
 from opensnitch import ui_pb2
 import time
 import ipaddress
@@ -14,8 +15,10 @@ import ipaddress
 from opensnitch.config import Config
 from opensnitch.nodes import Nodes
 from opensnitch.database import Database
+from opensnitch.database.enums import RuleFields, ConnFields
 from opensnitch.version import version
-from opensnitch.utils import Message, FileDialog
+from opensnitch.utils import Message, FileDialog, Icons, NetworkInterfaces
+from opensnitch.rules import Rule, Rules
 
 DIALOG_UI_PATH = "%s/../res/ruleseditor.ui" % os.path.dirname(sys.modules[__name__].__file__)
 class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
@@ -25,8 +28,11 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
     classB_net = "172\.1[6-9]\.\d+\.\d+|172\.2[0-9]\.\d+\.\d+|172\.3[0-1]+\.\d{1,3}\.\d{1,3}"
     classC_net = "192\.168\.\d{1,3}\.\d{1,3}"
     others_net = "127\.\d{1,3}\.\d{1,3}\.\d{1,3}|169\.254\.\d{1,3}\.\d{1,3}"
+    multinets = "2[32][23459]\.\d{1,3}\.\d{1,3}\.\d{1,3}"
+    MULTICAST_RANGE = "^(" + multinets + ")$"
     LAN_RANGES = "^(" + others_net + "|" + classC_net + "|" + classB_net + "|" + classA_net + "|::1|f[cde].*::.*)$"
     LAN_LABEL = "LAN"
+    MULTICAST_LABEL = "MULTICAST"
 
     ADD_RULE = 0
     EDIT_RULE = 1
@@ -36,20 +42,27 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
 
     def __init__(self, parent=None, _rule=None, appicon=None):
         super(RulesEditorDialog, self).__init__(parent)
-        QtWidgets.QDialog.__init__(self, parent, QtCore.Qt.WindowStaysOnTopHint)
 
         self._notifications_sent = {}
         self._nodes = Nodes.instance()
         self._db = Database.instance()
+        self._rules = Rules.instance()
         self._notification_callback.connect(self._cb_notification_callback)
         self._old_rule_name = None
 
         self.setupUi(self)
         self.setWindowIcon(appicon)
 
+        self.buttonBox.setStandardButtons(
+            QtWidgets.QDialogButtonBox.Help |
+            QtWidgets.QDialogButtonBox.Reset |
+            QtWidgets.QDialogButtonBox.Close |
+            QtWidgets.QDialogButtonBox.Save
+        )
+
         self.buttonBox.button(QtWidgets.QDialogButtonBox.Reset).clicked.connect(self._cb_reset_clicked)
         self.buttonBox.button(QtWidgets.QDialogButtonBox.Close).clicked.connect(self._cb_close_clicked)
-        self.buttonBox.button(QtWidgets.QDialogButtonBox.Apply).clicked.connect(self._cb_apply_clicked)
+        self.buttonBox.button(QtWidgets.QDialogButtonBox.Save).clicked.connect(self._cb_save_clicked)
         self.buttonBox.button(QtWidgets.QDialogButtonBox.Help).clicked.connect(self._cb_help_clicked)
         self.selectListButton.clicked.connect(self._cb_select_list_button_clicked)
         self.selectListRegexpButton.clicked.connect(self._cb_select_regexp_list_button_clicked)
@@ -58,23 +71,69 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         self.protoCheck.toggled.connect(self._cb_proto_check_toggled)
         self.procCheck.toggled.connect(self._cb_proc_check_toggled)
         self.cmdlineCheck.toggled.connect(self._cb_cmdline_check_toggled)
+        self.ifaceCheck.toggled.connect(self._cb_iface_check_toggled)
         self.dstPortCheck.toggled.connect(self._cb_dstport_check_toggled)
+        self.srcPortCheck.toggled.connect(self._cb_srcport_check_toggled)
         self.uidCheck.toggled.connect(self._cb_uid_check_toggled)
         self.pidCheck.toggled.connect(self._cb_pid_check_toggled)
+        self.srcIPCheck.toggled.connect(self._cb_srcip_check_toggled)
         self.dstIPCheck.toggled.connect(self._cb_dstip_check_toggled)
         self.dstHostCheck.toggled.connect(self._cb_dsthost_check_toggled)
         self.dstListsCheck.toggled.connect(self._cb_dstlists_check_toggled)
         self.dstListRegexpCheck.toggled.connect(self._cb_dstregexplists_check_toggled)
         self.dstListIPsCheck.toggled.connect(self._cb_dstiplists_check_toggled)
         self.dstListNetsCheck.toggled.connect(self._cb_dstnetlists_check_toggled)
+        self.uidCombo.currentIndexChanged.connect(self._cb_uid_combo_changed)
 
-        if QtGui.QIcon.hasThemeIcon("emblem-default") == False:
-            self.actionAllowRadio.setIcon(self.style().standardIcon(getattr(QtWidgets.QStyle, "SP_DialogApplyButton")))
-            self.actionDenyRadio.setIcon(self.style().standardIcon(getattr(QtWidgets.QStyle, "SP_DialogCancelButton")))
+        self._users_list = pwd.getpwall()
+
+        if QtGui.QIcon.hasThemeIcon("emblem-default"):
+            return
+
+        applyIcon = Icons.new("emblem-default")
+        denyIcon = Icons.new("emblem-important")
+        rejectIcon = Icons.new("window-close")
+        openIcon = Icons.new("document-open")
+        self.actionAllowRadio.setIcon(applyIcon)
+        self.actionDenyRadio.setIcon(denyIcon)
+        self.actionRejectRadio.setIcon(rejectIcon)
+        self.selectListButton.setIcon(openIcon)
+        self.selectListRegexpButton.setIcon(openIcon)
+        self.selectNetsListButton.setIcon(openIcon)
+        self.selectIPsListButton.setIcon(openIcon)
 
         if _rule != None:
             self._load_rule(rule=_rule)
 
+    def showEvent(self, event):
+        super(RulesEditorDialog, self).showEvent(event)
+
+        # save old combo values so we don't overwrite them here.
+        oldIface = self.ifaceCombo.currentText()
+        oldUid = self.uidCombo.currentText()
+        self.ifaceCombo.clear()
+        self.uidCombo.clear()
+        if self._nodes.is_local(self.nodesCombo.currentText()):
+            self.ifaceCombo.addItems(NetworkInterfaces.list().keys())
+            try:
+                for ip in NetworkInterfaces.list().values():
+                    if self.srcIPCombo.findText(ip) == -1:
+                        self.srcIPCombo.insertItem(0, ip)
+                    if self.dstIPCombo.findText(ip) == -1:
+                        self.dstIPCombo.insertItem(0, ip)
+
+                self._users_list = pwd.getpwall()
+                self.uidCombo.blockSignals(True);
+                for user in self._users_list:
+                    self.uidCombo.addItem("{0} ({1})".format(user[0], user[2]), user[2])
+                #self.uidCombo.setCurrentText("")
+            except Exception as e:
+                print("[ruleseditor] Error adding IPs:", e)
+            finally:
+                self.uidCombo.blockSignals(False);
+        self.ifaceCombo.setCurrentText(oldIface)
+        self.uidCombo.setCurrentText(oldUid)
+
     def _bool(self, s):
         return s == 'True'
 
@@ -121,15 +180,24 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         self.cmdlineLine.setEnabled(state)
         self.checkCmdlineRegexp.setEnabled(state)
 
+    def _cb_iface_check_toggled(self, state):
+        self.ifaceCombo.setEnabled(state)
+
     def _cb_dstport_check_toggled(self, state):
         self.dstPortLine.setEnabled(state)
 
+    def _cb_srcport_check_toggled(self, state):
+        self.srcPortLine.setEnabled(state)
+
     def _cb_uid_check_toggled(self, state):
-        self.uidLine.setEnabled(state)
+        self.uidCombo.setEnabled(state)
 
     def _cb_pid_check_toggled(self, state):
         self.pidLine.setEnabled(state)
 
+    def _cb_srcip_check_toggled(self, state):
+        self.srcIPCombo.setEnabled(state)
+
     def _cb_dstip_check_toggled(self, state):
         self.dstIPCombo.setEnabled(state)
 
@@ -152,6 +220,9 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         self.dstListNetsLine.setEnabled(state)
         self.selectNetsListButton.setEnabled(state)
 
+    def _cb_uid_combo_changed(self, index):
+        self.uidCombo.setCurrentText(str(self._users_list[index][2]))
+
     def _set_status_error(self, msg):
         self.statusLabel.setStyleSheet('color: red')
         self.statusLabel.setText(msg)
@@ -160,7 +231,7 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         self.statusLabel.setStyleSheet('color: green')
         self.statusLabel.setText(msg)
 
-    def _cb_apply_clicked(self):
+    def _cb_save_clicked(self):
         if self.nodesCombo.count() == 0:
             self._set_status_error(QC.translate("rules", "There're no nodes connected."))
             return
@@ -198,6 +269,8 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         if self.WORK_MODE == self.ADD_RULE:
             self.WORK_MODE = self.EDIT_RULE
 
+        self._rules.updated.emit(0)
+
     @QtCore.pyqtSlot(ui_pb2.NotificationReply)
     def _cb_notification_callback(self, reply):
         #print(self.LOG_TAG, "Rule notification received: ", reply.id, reply.code)
@@ -261,24 +334,25 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             self.statusLabel.setText(str(e))
             return False
 
-    def get_rule_from_records(self, records):
-        rule = ui_pb2.Rule(name=records.value(2))
-        rule.enabled = self._bool(records.value(3))
-        rule.precedence = self._bool(records.value(4))
-        rule.action = records.value(5)
-        rule.duration = records.value(6)
-        rule.operator.type = records.value(7)
-        rule.operator.sensitive = self._bool(records.value(8))
-        rule.operator.operand = records.value(9)
-        rule.operator.data = "" if records.value(10) == None else str(records.value(10))
-
-        return rule
+    def set_fields_from_connection(self, records):
+        self.nodesCombo.setCurrentText(records.value(ConnFields.Node))
+        self.protoCombo.setCurrentText(records.value(ConnFields.Protocol).upper())
+        self.srcIPCombo.setCurrentText(records.value(ConnFields.SrcIP))
+        self.dstIPCombo.setCurrentText(records.value(ConnFields.DstIP))
+        self.dstHostLine.setText(records.value(ConnFields.DstHost))
+        self.dstPortLine.setText(records.value(ConnFields.DstPort))
+        self.srcPortLine.setText(records.value(ConnFields.SrcPort))
+        self.uidCombo.setCurrentText(records.value(ConnFields.UID))
+        self.pidLine.setText(records.value(ConnFields.PID))
+        self.procLine.setText(records.value(ConnFields.Process))
+        self.cmdlineLine.setText(records.value(ConnFields.Cmdline))
 
     def _reset_state(self):
         self._old_rule_name = None
         self.rule = None
 
         self.ruleNameEdit.setText("")
+        self.ruleDescEdit.setPlainText("")
         self.statusLabel.setText("")
 
         self.actionDenyRadio.setChecked(True)
@@ -298,14 +372,23 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         self.cmdlineLine.setText("")
 
         self.uidCheck.setChecked(False)
-        self.uidLine.setText("")
+        self.uidCombo.setCurrentText("")
 
         self.pidCheck.setChecked(False)
         self.pidLine.setText("")
 
+        self.ifaceCheck.setChecked(False)
+        self.ifaceCombo.setCurrentText("")
+
         self.dstPortCheck.setChecked(False)
         self.dstPortLine.setText("")
 
+        self.srcPortCheck.setChecked(False)
+        self.srcPortLine.setText("")
+
+        self.srcIPCheck.setChecked(False)
+        self.srcIPCombo.setCurrentText("")
+
         self.dstIPCheck.setChecked(False)
         self.dstIPCombo.setCurrentText("")
 
@@ -333,8 +416,10 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             return False
 
         self.ruleNameEdit.setText(rule.name)
+        self.ruleDescEdit.setPlainText(rule.description)
         self.enableCheck.setChecked(rule.enabled)
         self.precedenceCheck.setChecked(rule.precedence)
+        self.nologCheck.setChecked(rule.nolog)
         if rule.action == Config.ACTION_DENY:
             self.actionDenyRadio.setChecked(True)
         elif rule.action == Config.ACTION_ALLOW:
@@ -360,72 +445,94 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
 
     def _load_rule_operator(self, operator):
         self.sensitiveCheck.setChecked(operator.sensitive)
-        if operator.operand == "protocol":
+        if operator.operand == Config.OPERAND_PROTOCOL:
             self.protoCheck.setChecked(True)
             self.protoCombo.setEnabled(True)
             self.protoCombo.setCurrentText(operator.data.upper())
 
-        if operator.operand == "process.path":
+        if operator.operand == Config.OPERAND_PROCESS_PATH:
             self.procCheck.setChecked(True)
             self.procLine.setEnabled(True)
             self.procLine.setText(operator.data)
             self.checkProcRegexp.setEnabled(True)
             self.checkProcRegexp.setChecked(operator.type == Config.RULE_TYPE_REGEXP)
 
-        if operator.operand == "process.command":
+        if operator.operand == Config.OPERAND_PROCESS_COMMAND:
             self.cmdlineCheck.setChecked(True)
             self.cmdlineLine.setEnabled(True)
             self.cmdlineLine.setText(operator.data)
             self.checkCmdlineRegexp.setEnabled(True)
             self.checkCmdlineRegexp.setChecked(operator.type == Config.RULE_TYPE_REGEXP)
 
-        if operator.operand == "user.id":
+        if operator.operand == Config.OPERAND_USER_ID:
             self.uidCheck.setChecked(True)
-            self.uidLine.setEnabled(True)
-            self.uidLine.setText(operator.data)
+            self.uidCombo.setEnabled(True)
+            self.uidCombo.setCurrentText(operator.data)
 
-        if operator.operand == "process.id":
+        if operator.operand == Config.OPERAND_PROCESS_ID:
             self.pidCheck.setChecked(True)
             self.pidLine.setEnabled(True)
             self.pidLine.setText(operator.data)
 
-        if operator.operand == "dest.port":
+        if operator.operand == Config.OPERAND_IFACE_OUT:
+            self.ifaceCheck.setChecked(True)
+            self.ifaceCombo.setEnabled(True)
+            self.ifaceCombo.setCurrentText(operator.data)
+
+        if operator.operand == Config.OPERAND_SOURCE_PORT:
+            self.srcPortCheck.setChecked(True)
+            self.srcPortLine.setEnabled(True)
+            self.srcPortLine.setText(operator.data)
+
+        if operator.operand == Config.OPERAND_DEST_PORT:
             self.dstPortCheck.setChecked(True)
             self.dstPortLine.setEnabled(True)
             self.dstPortLine.setText(operator.data)
 
-        if operator.operand == "dest.ip" or operator.operand == "dest.network":
+        if operator.operand == Config.OPERAND_SOURCE_IP or operator.operand == Config.OPERAND_SOURCE_NETWORK:
+            self.srcIPCheck.setChecked(True)
+            self.srcIPCombo.setEnabled(True)
+            if operator.data == self.LAN_RANGES:
+                self.srcIPCombo.setCurrentText(self.LAN_LABEL)
+            elif operator.data == self.MULTICAST_RANGE:
+                self.srcIPCombo.setCurrentText(self.MULTICAST_LABEL)
+            else:
+                self.srcIPCombo.setCurrentText(operator.data)
+
+        if operator.operand == Config.OPERAND_DEST_IP or operator.operand == Config.OPERAND_DEST_NETWORK:
             self.dstIPCheck.setChecked(True)
             self.dstIPCombo.setEnabled(True)
             if operator.data == self.LAN_RANGES:
                 self.dstIPCombo.setCurrentText(self.LAN_LABEL)
+            elif operator.data == self.MULTICAST_RANGE:
+                self.dstIPCombo.setCurrentText(self.MULTICAST_LABEL)
             else:
                 self.dstIPCombo.setCurrentText(operator.data)
 
-        if operator.operand == "dest.host":
+        if operator.operand == Config.OPERAND_DEST_HOST:
             self.dstHostCheck.setChecked(True)
             self.dstHostLine.setEnabled(True)
             self.dstHostLine.setText(operator.data)
 
-        if operator.operand == "lists.domains":
+        if operator.operand == Config.OPERAND_LIST_DOMAINS:
             self.dstListsCheck.setChecked(True)
             self.dstListsCheck.setEnabled(True)
             self.dstListsLine.setText(operator.data)
             self.selectListButton.setEnabled(True)
 
-        if operator.operand == "lists.domains_regexp":
+        if operator.operand == Config.OPERAND_LIST_DOMAINS_REGEXP:
             self.dstListRegexpCheck.setChecked(True)
             self.dstListRegexpCheck.setEnabled(True)
             self.dstRegexpListsLine.setText(operator.data)
             self.selectListRegexpButton.setEnabled(True)
 
-        if operator.operand == "lists.ips":
+        if operator.operand == Config.OPERAND_LIST_IPS:
             self.dstListIPsCheck.setChecked(True)
             self.dstListIPsCheck.setEnabled(True)
             self.dstListIPsLine.setText(operator.data)
             self.selectIPsListButton.setEnabled(True)
 
-        if operator.operand == "lists.nets":
+        if operator.operand == Config.OPERAND_LIST_NETS:
             self.dstListNetsCheck.setChecked(True)
             self.dstListNetsCheck.setEnabled(True)
             self.dstListNetsLine.setText(operator.data)
@@ -458,11 +565,13 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         return True
 
     def _insert_rule_to_db(self, node_addr):
+        # the order of the fields doesn't matter here, as long as we use the
+        # name of the field.
         self._db.insert("rules",
-            "(time, node, name, enabled, precedence, action, duration, operator_type, operator_sensitive, operator_operand, operator_data)",
+            "(time, node, name, description, enabled, precedence, nolog, action, duration, operator_type, operator_sensitive, operator_operand, operator_data)",
                 (datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
-                    node_addr, self.rule.name,
-                    str(self.rule.enabled), str(self.rule.precedence),
+                    node_addr, self.rule.name, self.rule.description,
+                    str(self.rule.enabled), str(self.rule.precedence), str(self.rule.nolog),
                     self.rule.action, self.rule.duration, self.rule.operator.type,
                     str(self.rule.operator.sensitive), self.rule.operator.operand, self.rule.operator.data),
                 action_on_conflict="REPLACE")
@@ -519,8 +628,10 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         """
         self.rule = ui_pb2.Rule()
         self.rule.name = self.ruleNameEdit.text()
+        self.rule.description = self.ruleDescEdit.toPlainText()
         self.rule.enabled = self.enableCheck.isChecked()
         self.rule.precedence = self.precedenceCheck.isChecked()
+        self.rule.nolog = self.nologCheck.isChecked()
         self.rule.operator.type = Config.RULE_TYPE_SIMPLE
         self.rule.action = Config.ACTION_DENY
         if self.actionAllowRadio.isChecked():
@@ -537,12 +648,12 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             if self.protoCombo.currentText() == "":
                 return False, QC.translate("rules", "protocol can not be empty, or uncheck it")
 
-            self.rule.operator.operand = "protocol"
+            self.rule.operator.operand = Config.OPERAND_PROTOCOL
             self.rule.operator.data = self.protoCombo.currentText()
             rule_data.append(
                     {
                         "type": Config.RULE_TYPE_SIMPLE,
-                        "operand": "protocol",
+                        "operand": Config.OPERAND_PROTOCOL,
                         "data": self.protoCombo.currentText().lower(),
                         "sensitive": self.sensitiveCheck.isChecked()
                         })
@@ -555,12 +666,12 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             if self.procLine.text() == "":
                 return False, QC.translate("rules", "process path can not be empty")
 
-            self.rule.operator.operand = "process.path"
+            self.rule.operator.operand = Config.OPERAND_PROCESS_PATH
             self.rule.operator.data = self.procLine.text()
             rule_data.append(
                     {
                         "type": Config.RULE_TYPE_SIMPLE,
-                        "operand": "process.path",
+                        "operand": Config.OPERAND_PROCESS_PATH,
                         "data": self.procLine.text(),
                         "sensitive": self.sensitiveCheck.isChecked()
                         })
@@ -573,12 +684,12 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             if self.cmdlineLine.text() == "":
                 return False, QC.translate("rules", "command line can not be empty")
 
-            self.rule.operator.operand = "process.command"
+            self.rule.operator.operand = Config.OPERAND_PROCESS_COMMAND
             self.rule.operator.data = self.cmdlineLine.text()
             rule_data.append(
                     {
                         'type': Config.RULE_TYPE_SIMPLE,
-                        'operand': 'process.command',
+                        'operand': Config.OPERAND_PROCESS_COMMAND,
                         'data': self.cmdlineLine.text(),
                         "sensitive": self.sensitiveCheck.isChecked()
                         })
@@ -587,16 +698,52 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
                 if self._is_valid_regex(self.cmdlineLine.text()) == False:
                     return False, QC.translate("rules", "Command line regexp error")
 
+        if self.ifaceCheck.isChecked():
+            if self.ifaceCombo.currentText() == "":
+                return False, QC.translate("rules", "Network interface can not be empty")
+
+            self.rule.operator.operand = Config.OPERAND_IFACE_OUT
+            self.rule.operator.data = self.ifaceCombo.currentText()
+            rule_data.append(
+                    {
+                        'type': Config.RULE_TYPE_SIMPLE,
+                        'operand': Config.OPERAND_IFACE_OUT,
+                        'data': self.ifaceCombo.currentText(),
+                        "sensitive": self.sensitiveCheck.isChecked()
+                        })
+            if self._is_regex(self.ifaceCombo.currentText()):
+                rule_data[len(rule_data)-1]['type'] = Config.RULE_TYPE_REGEXP
+                if self._is_valid_regex(self.ifaceCombo.currentText()) == False:
+                    return False, QC.translate("rules", "Network interface regexp error")
+
+        if self.srcPortCheck.isChecked():
+            if self.srcPortLine.text() == "":
+                return False, QC.translate("rules", "Source port can not be empty")
+
+            self.rule.operator.operand = Config.OPERAND_SOURCE_PORT
+            self.rule.operator.data = self.srcPortLine.text()
+            rule_data.append(
+                    {
+                        'type': Config.RULE_TYPE_SIMPLE,
+                        'operand': Config.OPERAND_SOURCE_PORT,
+                        'data': self.srcPortLine.text(),
+                        "sensitive": self.sensitiveCheck.isChecked()
+                        })
+            if self._is_regex(self.srcPortLine.text()):
+                rule_data[len(rule_data)-1]['type'] = Config.RULE_TYPE_REGEXP
+                if self._is_valid_regex(self.srcPortLine.text()) == False:
+                    return False, QC.translate("rules", "Source port regexp error")
+
         if self.dstPortCheck.isChecked():
             if self.dstPortLine.text() == "":
                 return False, QC.translate("rules", "Dest port can not be empty")
 
-            self.rule.operator.operand = "dest.port"
+            self.rule.operator.operand = Config.OPERAND_DEST_PORT
             self.rule.operator.data = self.dstPortLine.text()
             rule_data.append(
                     {
                         'type': Config.RULE_TYPE_SIMPLE,
-                        'operand': 'dest.port',
+                        'operand': Config.OPERAND_DEST_PORT,
                         'data': self.dstPortLine.text(),
                         "sensitive": self.sensitiveCheck.isChecked()
                         })
@@ -609,12 +756,12 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             if self.dstHostLine.text() == "":
                 return False, QC.translate("rules", "Dest host can not be empty")
 
-            self.rule.operator.operand = "dest.host"
+            self.rule.operator.operand = Config.OPERAND_DEST_HOST
             self.rule.operator.data = self.dstHostLine.text()
             rule_data.append(
                     {
                         'type': Config.RULE_TYPE_SIMPLE,
-                        'operand': 'dest.host',
+                        'operand': Config.OPERAND_DEST_HOST,
                         'data': self.dstHostLine.text(),
                         "sensitive": self.sensitiveCheck.isChecked()
                         })
@@ -623,6 +770,44 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
                 if self._is_valid_regex(self.dstHostLine.text()) == False:
                     return False, QC.translate("rules", "Dst host regexp error")
 
+        if self.srcIPCheck.isChecked():
+            if self.srcIPCombo.currentText() == "":
+                return False, QC.translate("rules", "Source IP/Network can not be empty")
+
+            srcIPtext = self.srcIPCombo.currentText()
+
+            if srcIPtext == self.LAN_LABEL:
+                self.rule.operator.operand = Config.OPERAND_SOURCE_IP
+                self.rule.operator.type = Config.RULE_TYPE_REGEXP
+                srcIPtext = self.LAN_RANGES
+            elif srcIPtext == self.MULTICAST_LABEL:
+                self.rule.operator.operand = Config.OPERAND_SOURCE_IP
+                self.rule.operator.type = Config.RULE_TYPE_REGEXP
+                srcIPtext = self.MULTICAST_RANGE
+            else:
+                try:
+                    if type(ipaddress.ip_address(self.srcIPCombo.currentText())) == ipaddress.IPv4Address \
+                    or type(ipaddress.ip_address(self.srcIPCombo.currentText())) == ipaddress.IPv6Address:
+                        self.rule.operator.operand = Config.OPERAND_SOURCE_IP
+                        self.rule.operator.type = Config.RULE_TYPE_SIMPLE
+                except Exception:
+                    self.rule.operator.operand = Config.OPERAND_SOURCE_NETWORK
+                    self.rule.operator.type = Config.RULE_TYPE_NETWORK
+
+                if self._is_regex(srcIPtext):
+                    self.rule.operator.operand = Config.OPERAND_SOURCE_IP
+                    self.rule.operator.type = Config.RULE_TYPE_REGEXP
+                    if self._is_valid_regex(self.srcIPCombo.currentText()) == False:
+                        return False, QC.translate("rules", "Source IP regexp error")
+
+            rule_data.append(
+                    {
+                        'type': self.rule.operator.type,
+                        'operand': self.rule.operator.operand,
+                        'data': srcIPtext,
+                        "sensitive": self.sensitiveCheck.isChecked()
+                        })
+
         if self.dstIPCheck.isChecked():
             if self.dstIPCombo.currentText() == "":
                 return False, QC.translate("rules", "Dest IP/Network can not be empty")
@@ -630,21 +815,25 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             dstIPtext = self.dstIPCombo.currentText()
 
             if dstIPtext == self.LAN_LABEL:
-                self.rule.operator.operand = "dest.ip"
+                self.rule.operator.operand = Config.OPERAND_DEST_IP
                 self.rule.operator.type = Config.RULE_TYPE_REGEXP
                 dstIPtext = self.LAN_RANGES
+            elif dstIPtext == self.MULTICAST_LABEL:
+                self.rule.operator.operand = Config.OPERAND_DEST_IP
+                self.rule.operator.type = Config.RULE_TYPE_REGEXP
+                dstIPtext = self.MULTICAST_RANGE
             else:
                 try:
                     if type(ipaddress.ip_address(self.dstIPCombo.currentText())) == ipaddress.IPv4Address \
                     or type(ipaddress.ip_address(self.dstIPCombo.currentText())) == ipaddress.IPv6Address:
-                        self.rule.operator.operand = "dest.ip"
+                        self.rule.operator.operand = Config.OPERAND_DEST_IP
                         self.rule.operator.type = Config.RULE_TYPE_SIMPLE
                 except Exception:
-                    self.rule.operator.operand = "dest.network"
+                    self.rule.operator.operand = Config.OPERAND_DEST_NETWORK
                     self.rule.operator.type = Config.RULE_TYPE_NETWORK
 
                 if self._is_regex(dstIPtext):
-                    self.rule.operator.operand = "dest.ip"
+                    self.rule.operator.operand = Config.OPERAND_DEST_IP
                     self.rule.operator.type = Config.RULE_TYPE_REGEXP
                     if self._is_valid_regex(self.dstIPCombo.currentText()) == False:
                         return False, QC.translate("rules", "Dst IP regexp error")
@@ -658,33 +847,33 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
                         })
 
         if self.uidCheck.isChecked():
-            if self.uidLine.text() == "":
+            if self.uidCombo.currentText() == "":
                 return False, QC.translate("rules", "User ID can not be empty")
 
-            self.rule.operator.operand = "user.id"
-            self.rule.operator.data = self.uidLine.text()
+            self.rule.operator.operand = Config.OPERAND_USER_ID
+            self.rule.operator.data = self.uidCombo.currentText()
             rule_data.append(
                     {
                         'type': Config.RULE_TYPE_SIMPLE,
-                        'operand': 'user.id',
-                        'data': self.uidLine.text(),
+                        'operand': Config.OPERAND_USER_ID,
+                        'data': self.uidCombo.currentText(),
                         "sensitive": self.sensitiveCheck.isChecked()
                         })
-            if self._is_regex(self.uidLine.text()):
+            if self._is_regex(self.uidCombo.currentText()):
                 rule_data[len(rule_data)-1]['type'] = Config.RULE_TYPE_REGEXP
-                if self._is_valid_regex(self.uidLine.text()) == False:
+                if self._is_valid_regex(self.uidCombo.currentText()) == False:
                     return False, QC.translate("rules", "User ID regexp error")
 
         if self.pidCheck.isChecked():
             if self.pidLine.text() == "":
                 return False, QC.translate("rules", "PID field can not be empty")
 
-            self.rule.operator.operand = "process.id"
+            self.rule.operator.operand = Config.OPERAND_PROCESS_ID
             self.rule.operator.data = self.pidLine.text()
             rule_data.append(
                     {
                         'type': Config.RULE_TYPE_SIMPLE,
-                        'operand': 'process.id',
+                        'operand': Config.OPERAND_PROCESS_ID,
                         'data': self.pidLine.text(),
                         "sensitive": self.sensitiveCheck.isChecked()
                         })
@@ -700,11 +889,11 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
                 return False, QC.translate("rules", "Lists field must be a directory")
 
             self.rule.operator.type = Config.RULE_TYPE_LISTS
-            self.rule.operator.operand = "lists.domains"
+            self.rule.operator.operand = Config.OPERAND_LIST_DOMAINS
             rule_data.append(
                     {
                         'type': Config.RULE_TYPE_LISTS,
-                        'operand': 'lists.domains',
+                        'operand': Config.OPERAND_LIST_DOMAINS,
                         'data': self.dstListsLine.text(),
                         'sensitive': self.sensitiveCheck.isChecked()
                         })
@@ -717,11 +906,11 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
                 return False, QC.translate("rules", "Lists field must be a directory")
 
             self.rule.operator.type = Config.RULE_TYPE_LISTS
-            self.rule.operator.operand = "lists.domains_regexp"
+            self.rule.operator.operand = Config.OPERAND_LIST_DOMAINS_REGEXP
             rule_data.append(
                     {
                         'type': Config.RULE_TYPE_LISTS,
-                        'operand': 'lists.domains_regexp',
+                        'operand': Config.OPERAND_LIST_DOMAINS_REGEXP,
                         'data': self.dstRegexpListsLine.text(),
                         'sensitive': self.sensitiveCheck.isChecked()
                         })
@@ -734,11 +923,11 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
                 return False, QC.translate("rules", "Lists field must be a directory")
 
             self.rule.operator.type = Config.RULE_TYPE_LISTS
-            self.rule.operator.operand = "lists.nets"
+            self.rule.operator.operand = Config.OPERAND_LIST_NETS
             rule_data.append(
                     {
                         'type': Config.RULE_TYPE_LISTS,
-                        'operand': 'lists.nets',
+                        'operand': Config.OPERAND_LIST_NETS,
                         'data': self.dstListNetsLine.text(),
                         'sensitive': self.sensitiveCheck.isChecked()
                         })
@@ -752,11 +941,11 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
                 return False, QC.translate("rules", "Lists field must be a directory")
 
             self.rule.operator.type = Config.RULE_TYPE_LISTS
-            self.rule.operator.operand = "lists.ips"
+            self.rule.operator.operand = Config.OPERAND_LIST_IPS
             rule_data.append(
                     {
                         'type': Config.RULE_TYPE_LISTS,
-                        'operand': 'lists.ips',
+                        'operand': Config.OPERAND_LIST_IPS,
                         'data': self.dstListIPsLine.text(),
                         'sensitive': self.sensitiveCheck.isChecked()
                         })
@@ -790,7 +979,7 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         self.WORK_MODE = self.EDIT_RULE
         self._reset_state()
 
-        self.rule = self.get_rule_from_records(records)
+        self.rule = Rule.new_from_records(records)
         if self.rule.operator.type not in Config.RulesTypes:
             Message.ok(QC.translate("rules", "<b>Rule not supported</b>"),
                        QC.translate("rules", "This type of rule ({0}) is not supported by version {1}".format(self.rule.operator.type, version)),
@@ -798,7 +987,7 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             self.hide()
             return
 
-        self._old_rule_name = records.value(2)
+        self._old_rule_name = records.value(RuleFields.Name)
 
         if self._load_rule(addr=_addr, rule=self.rule):
             self.show()
@@ -808,3 +997,22 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         self._reset_state()
         self._load_nodes()
         self.show()
+
+    def new_rule_from_connection(self, coltime):
+        self.WORK_MODE = self.ADD_RULE
+        self._reset_state()
+        self._load_nodes()
+
+        try:
+            records = self._db.get_connection_by_field("time", coltime)
+            if records.next() == False:
+                print(self.LOG_TAG, "error loading connection fields by time: {0}".format(coltime))
+                return False
+
+            self.set_fields_from_connection(records)
+            self.show()
+        except Exception as e:
+            print(self.LOG_TAG, "exception creating new rule from connection:", e)
+            return False
+
+        return True
