@@ -1,4 +1,5 @@
 from PyQt5 import QtWidgets, QtGui, QtCore
+from PyQt5.QtCore import QCoreApplication as QC
 
 from datetime import datetime, timedelta
 from threading import Thread, Lock, Event
@@ -6,6 +7,7 @@ import grpc
 import os
 import sys
 import json
+import copy
 
 path = os.path.abspath(os.path.dirname(__file__))
 sys.path.append(path)
@@ -17,21 +19,23 @@ from opensnitch.dialogs.prompt import PromptDialog
 from opensnitch.dialogs.stats import StatsDialog
 
 from opensnitch.notifications import DesktopNotifications
+from opensnitch.firewall import Rules as FwRules
 from opensnitch.nodes import Nodes
 from opensnitch.config import Config
 from opensnitch.version import version
 from opensnitch.database import Database
-from opensnitch.utils import Utils, CleanerTask
-from opensnitch.utils import Message
+from opensnitch.utils import Utils, CleanerTask, Themes
+from opensnitch.utils import Message, languages
 
 class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
     _new_remote_trigger = QtCore.pyqtSignal(str, ui_pb2.PingRequest)
     _node_actions_trigger = QtCore.pyqtSignal(dict)
     _update_stats_trigger = QtCore.pyqtSignal(str, str, ui_pb2.PingRequest)
+    _add_alert_trigger = QtCore.pyqtSignal(str, str, ui_pb2.Alert)
     _version_warning_trigger = QtCore.pyqtSignal(str, str)
     _status_change_trigger = QtCore.pyqtSignal(bool)
     _notification_callback = QtCore.pyqtSignal(ui_pb2.NotificationReply)
-    _show_message_trigger = QtCore.pyqtSignal(str, str, int)
+    _show_message_trigger = QtCore.pyqtSignal(str, str, int, int)
 
     # .desktop filename located under /usr/share/applications/
     DESKTOP_FILENAME = "opensnitch_ui.desktop"
@@ -84,6 +88,9 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
         self._remote_lock = Lock()
         self._remote_stats = {}
 
+        self.translator = None
+        self._init_translation()
+        self._themes = Themes()
         self._desktop_notifications = DesktopNotifications()
         self._setup_interfaces()
         self._setup_icons()
@@ -93,6 +100,7 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
         self._setup_slots()
 
         self._nodes = Nodes.instance()
+        self._nodes.reset_status()
 
         self._last_stats = {}
         self._last_items = {
@@ -108,6 +116,12 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
         self._cleaner = None
         if self._cfg.getBool(Config.DEFAULT_DB_PURGE_OLDEST):
             self._start_db_cleaner()
+        self._cfg.setRulesDurationFilter(
+            self._cfg.getBool(self._cfg.DEFAULT_IGNORE_RULES),
+            self._cfg.getInt(self._cfg.DEFAULT_IGNORE_TEMPORARY_RULES)
+        )
+        if self._cfg.getBool(self._cfg.DEFAULT_IGNORE_RULES):
+            self._nodes.delete_rule_by_field(Config.DURATION_FIELD, Config.RULES_DURATION_FILTER)
 
     # https://gist.github.com/pklaus/289646
     def _setup_interfaces(self):
@@ -125,10 +139,12 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
         self._new_remote_trigger.connect(self._on_new_remote)
         self._node_actions_trigger.connect(self._on_node_actions)
         self._update_stats_trigger.connect(self._on_update_stats)
+        self._add_alert_trigger.connect(self._on_new_alert)
         self._status_change_trigger.connect(self._on_status_changed)
         self._stats_dialog._shown_trigger.connect(self._on_stats_dialog_shown)
         self._stats_dialog._status_changed_trigger.connect(self._on_stats_status_changed)
         self._stats_dialog.settings_saved.connect(self._on_settings_saved)
+        self._stats_dialog.close_trigger.connect(self._on_close)
         self._show_message_trigger.connect(self._show_systray_message)
 
     def _setup_icons(self):
@@ -178,10 +194,29 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
         gui = self._stats_dialog
         def __show_gui():
             if not tray.isSystemTrayAvailable():
+                self._show_systray_msg_error()
                 gui.show()
 
         QtCore.QTimer.singleShot(10000, __show_gui)
 
+    def _show_systray_msg_error(self):
+        print("")
+        print("WARNING: system tray not available. On GNOME you need the extension gnome-shell-extension-appindicator.")
+        print("\tRead more:", Config.HELP_SYSTRAY_WARN)
+        print("")
+
+        hide_msg = self._cfg.getBool(Config.DEFAULT_HIDE_SYSTRAY_WARN)
+        if hide_msg:
+            return
+        self._desktop_notifications.show(
+            QC.translate("stats", "WARNING"),
+            QC.translate("stats", """System tray not available. Read more:
+{0}
+""".format(Config.HELP_SYSTRAY_WARN)),
+            os.path.join(self._path, "res/icon-white.svg")
+        )
+        self._cfg.setSettings(Config.DEFAULT_HIDE_SYSTRAY_WARN, True)
+
     def _on_tray_icon_activated(self, reason):
         if reason == QtWidgets.QSystemTrayIcon.Trigger or reason == QtWidgets.QSystemTrayIcon.MiddleClick:
             if self._stats_dialog.isVisible() and not self._stats_dialog.isMinimized():
@@ -237,6 +272,51 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
         is_local_request = self._is_local_request(proto, addr)
         self._stats_dialog.update(is_local_request, request.stats, main_need_refresh or details_need_refresh)
 
+    @QtCore.pyqtSlot(str, str, ui_pb2.Alert)
+    def _on_new_alert(self, proto, addr, alert):
+        try:
+            is_local = self._is_local_request(proto, addr)
+
+            icon = QtWidgets.QSystemTrayIcon.Information
+            _title = QtCore.QCoreApplication.translate("messages", "Info")
+            atype = "INFO"
+            if alert.type == ui_pb2.Alert.ERROR:
+                atype = "ERROR"
+                _title = QtCore.QCoreApplication.translate("messages", "Error")
+                icon = QtWidgets.QSystemTrayIcon.Critical
+            if alert.type == ui_pb2.Alert.WARNING:
+                atype = "WARNING"
+                _title = QtCore.QCoreApplication.translate("messages", "Warning")
+                icon = QtWidgets.QSystemTrayIcon.Warning
+
+            body = ""
+            what = "GENERIC"
+            if alert.what == ui_pb2.Alert.GENERIC:
+                body = alert.text
+            elif alert.what == ui_pb2.Alert.KERNEL_EVENT:
+                body = "%s\n%s" % (alert.text, alert.proc.path)
+                what = "KERNEL EVENT"
+            if is_local is False:
+                body = "node: {0}:{1}\n\n{2}\n{3}".format(proto, addr, alert.text, alert.proc.path)
+
+            if alert.action == ui_pb2.Alert.SHOW_ALERT:
+
+                urgency = DesktopNotifications.URGENCY_NORMAL
+                if alert.priority == ui_pb2.Alert.LOW:
+                    urgency = DesktopNotifications.URGENCY_LOW
+                elif alert.priority == ui_pb2.Alert.HIGH:
+                    urgency = DesktopNotifications.URGENCY_CRITICAL
+
+                self._show_message_trigger.emit(_title, body, icon, urgency)
+
+            else:
+                print("PostAlert() unknown alert action:", alert.action)
+
+
+        except Exception as e:
+            print("PostAlert() exception:", e)
+            return ui_pb2.MsgResponse(id=1)
+
     @QtCore.pyqtSlot(str, ui_pb2.PingRequest)
     def _on_new_remote(self, addr, request):
         self._remote_stats[addr] = {
@@ -268,13 +348,27 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
     def _on_remote_stats_menu(self, address):
         self._remote_stats[address]['dialog'].show()
 
-    @QtCore.pyqtSlot(str, str, int)
-    def _show_systray_message(self, title, body, icon):
+    @QtCore.pyqtSlot(str, str, int, int)
+    def _show_systray_message(self, title, body, icon, urgency):
+        def callback_open_clicked(notifObject, action):
+            if action == DesktopNotifications.ACTION_ID_OPEN:
+                self._stats_dialog.show()
+                #self._stats_dialog.raise()
+                self._stats_dialog.activateWindow()
+
         if self._desktop_notifications.are_enabled():
             timeout = self._cfg.getInt(Config.DEFAULT_TIMEOUT_KEY, 15)
 
             if self._desktop_notifications.is_available() and self._cfg.getInt(Config.NOTIFICATIONS_TYPE, 1) == Config.NOTIFICATION_TYPE_SYSTEM:
-                self._desktop_notifications.show(title, body, os.path.join(self._path, "res/icon-white.svg"))
+                try:
+                    self._desktop_notifications.show(
+                        title,
+                        body,
+                        os.path.join(self._path, "res/icon-white.svg"),
+                        callback=callback_open_clicked
+                    )
+                except:
+                    self._tray.showMessage(title, body, icon, timeout * 1000)
             else:
                 self._tray.showMessage(title, body, icon, timeout * 1000)
 
@@ -293,6 +387,17 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
         elif self._cfg.getBool(Config.DEFAULT_DB_PURGE_OLDEST) == False and self._cleaner != None:
             self._stop_db_cleaner()
 
+        theme_idx, theme_name = self._themes.get_saved_theme()
+        if theme_idx > 0:
+            self._themes.load_theme(self._app)
+
+    def _init_translation(self):
+        if self.translator:
+            self._app.removeTranslator(self.translator)
+        saved_lang = self._cfg.getSettings(Config.DEFAULT_LANGUAGE)
+        self.translator = languages.init(saved_lang)
+        self._app.installTranslator(self.translator)
+
     def _stop_db_cleaner(self):
         if self._cleaner != None:
             self._cleaner.stop()
@@ -404,7 +509,7 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
                 if addr in self._last_items[table]:
                     del self._last_items[table][addr]
 
-            self._nodes.update(proto, addr, Nodes.OFFLINE)
+            self._nodes.update(peer, Nodes.OFFLINE)
             self._nodes.delete(peer)
             self._stats_dialog.update(True, None, True)
         except Exception as e:
@@ -418,7 +523,8 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
                 print("populate_stats() db None")
                 return main_need_refresh, details_need_refresh
 
-            _node = self._nodes.get_node(proto+":"+addr)
+            peer = proto+":"+addr
+            _node = self._nodes.get_node(peer)
             if _node == None:
                 return main_need_refresh, details_need_refresh
 
@@ -428,7 +534,7 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
             db.insert("nodes",
                     "(addr, status, hostname, daemon_version, daemon_uptime, " \
                             "daemon_rules, cons, cons_dropped, version, last_connection)",
-                            (addr, Nodes.ONLINE, hostname, stats.daemon_version, str(timedelta(seconds=stats.uptime)),
+                            (peer, Nodes.ONLINE, hostname, stats.daemon_version, str(timedelta(seconds=stats.uptime)),
                             stats.rules, stats.connections, stats.dropped,
                             version, datetime.now().strftime("%Y-%m-%d %H:%M:%S")))
 
@@ -442,7 +548,7 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
                 main_need_refresh=True
                 db.insert("connections",
                         "(time, node, action, protocol, src_ip, src_port, dst_ip, dst_host, dst_port, uid, pid, process, process_args, process_cwd, rule)",
-                        (str(datetime.fromtimestamp(event.unixnano/1000000000)), "%s:%s" % (proto, addr), event.rule.action,
+                        (str(datetime.fromtimestamp(event.unixnano/1000000000)), peer, event.rule.action,
                             event.connection.protocol, event.connection.src_ip, str(event.connection.src_port),
                             event.connection.dst_ip, event.connection.dst_host, str(event.connection.dst_port),
                             str(event.connection.user_id), str(event.connection.process_id),
@@ -453,7 +559,7 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
                 self._nodes.update_rule_time(
                     str(datetime.fromtimestamp(event.unixnano/1000000000)),
                     event.rule.name,
-                    "%s:%s" % (proto, addr)
+                    peer
                 )
             db.commit()
 
@@ -514,25 +620,34 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
         return need_refresh
 
     def _overwrite_nodes_config(self, node_config):
+        """Overwrite daemon's DefaultAction value, with the one defined by the GUI.
+        It'll only be valid while the daemon is connected to the GUI (it's not saved to disk).
+        """
+        newconf = copy.deepcopy(node_config)
         _default_action = self._cfg.getInt(self._cfg.DEFAULT_ACTION_KEY)
-        temp_cfg = json.loads(node_config)
         try:
+            temp_cfg = json.loads(newconf.config)
             if _default_action == Config.ACTION_DENY_IDX:
                 temp_cfg['DefaultAction'] = Config.ACTION_DENY
             else:
                 temp_cfg['DefaultAction'] = Config.ACTION_ALLOW
 
-            node_config = json.dumps(temp_cfg)
+            newconf.config = json.dumps(temp_cfg)
         except Exception as e:
             print("error parsing node's configuration:", e)
+            return node_config
 
-        return node_config
+        return newconf
 
     @QtCore.pyqtSlot(dict)
     def _on_node_actions(self, kwargs):
         if kwargs['action'] == self.NODE_ADD:
-            n = self._nodes.add(kwargs['peer'], kwargs['node_config'])
+            n, addr = self._nodes.add(kwargs['peer'], kwargs['node_config'])
             if n != None:
+                self._nodes.add_fw_rules(
+                    addr,
+                    FwRules.to_dict(kwargs['node_config'].systemFirewall.SystemRules)
+                )
                 self._status_change_trigger.emit(True)
                 # if there're more than one node, we can't update the status
                 # based on the fw status, only if the daemon is running or not
@@ -545,15 +660,22 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
             proto, addr = self._get_peer(kwargs['peer'])
             self._nodes.add_rule((datetime.now().strftime("%Y-%m-%d %H:%M:%S")),
                                  "{0}:{1}".format(proto, addr),
-                                rule.name, str(rule.enabled), str(rule.precedence), rule.action, rule.duration,
-                                rule.operator.type, str(rule.operator.sensitive), rule.operator.operand,
-                                rule.operator.data)
+                                 rule.name, rule.description, str(rule.enabled),
+                                 str(rule.precedence), str(rule.nolog), rule.action, rule.duration,
+                                 rule.operator.type, str(rule.operator.sensitive), rule.operator.operand,
+                                 rule.operator.data)
         elif kwargs['action'] == self.DELETE_RULE:
             self._db.delete_rule(kwargs['name'], kwargs['addr'])
 
         elif kwargs['action'] == self.NODE_DELETE:
             self._delete_node(kwargs['peer'])
 
+
+    def PostAlert(self, alert, context):
+        proto, addr = self._get_peer(context.peer())
+        self._add_alert_trigger.emit(proto, addr, alert)
+        return ui_pb2.MsgResponse(id=0)
+
     def Ping(self, request, context):
         try:
             self._last_ping = datetime.now()
@@ -604,7 +726,8 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
             self._show_message_trigger.emit(_title,
                                             "{0} action applied {1}\nCommand line: {2}"
                                             .format(rule.action, node_text, " ".join(request.process_args)),
-                                            QtWidgets.QSystemTrayIcon.NoIcon)
+                                            QtWidgets.QSystemTrayIcon.NoIcon,
+                                            DesktopNotifications.URGENCY_NORMAL)
 
 
         if rule.duration in Config.RULES_DURATION_FILTER:
@@ -636,6 +759,7 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
         # if the exit mark is set, don't accept new connections.
         # db vacuum operation may take a lot of time to complete.
         if self._exit:
+            context.cancel()
             return
         try:
             self._node_actions_trigger.emit({
@@ -652,14 +776,16 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
                 self._show_message_trigger.emit(
                     QtCore.QCoreApplication.translate("stats", "New node connected"),
                     "({0})".format(context.peer()),
-                    QtWidgets.QSystemTrayIcon.Information)
+                    QtWidgets.QSystemTrayIcon.Information,
+                    DesktopNotifications.URGENCY_LOW
+                )
         except Exception as e:
             print("[Notifications] exception adding new node:", e)
             context.cancel()
 
-        node_config.config = self._overwrite_nodes_config(node_config.config)
+        newconf = self._overwrite_nodes_config(node_config)
 
-        return node_config
+        return newconf
 
     def Notifications(self, node_iter, context):
         """
@@ -691,8 +817,9 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
 
             if self._is_local_request(proto, addr) == False:
                 self._show_message_trigger.emit("node exited",
-                                    "({0})".format(context.peer()),
-                                    QtWidgets.QSystemTrayIcon.Information)
+                                                "({0})".format(context.peer()),
+                                                QtWidgets.QSystemTrayIcon.Information,
+                                                DesktopNotifications.URGENCY_LOW)
 
         context.add_callback(_on_client_closed)
 
